package com.boman.system.service.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.alibaba.fastjson.JSONObject;
import com.boman.common.core.utils.DateUtils;
import com.boman.common.core.utils.obj.ObjectUtils;
import com.boman.domain.SysDept;
import com.boman.system.service.ISysDeptService;
import com.boman.domain.dto.AjaxResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.boman.domain.constant.UserConstants;
import com.boman.common.core.exception.CustomException;
import com.boman.common.core.utils.SecurityUtils;
import com.boman.common.core.utils.StringUtils;
import com.boman.common.datascope.annotation.DataScope;
import com.boman.domain.SysRole;
import com.boman.domain.SysUser;
import com.boman.system.domain.SysPost;
import com.boman.system.domain.SysUserPost;
import com.boman.system.domain.SysUserRole;
import com.boman.system.mapper.SysPostMapper;
import com.boman.system.mapper.SysRoleMapper;
import com.boman.system.mapper.SysUserMapper;
import com.boman.system.mapper.SysUserPostMapper;
import com.boman.system.mapper.SysUserRoleMapper;
import com.boman.system.service.ISysConfigService;
import com.boman.system.service.ISysUserService;

import javax.annotation.Resource;
import javax.validation.constraints.Size;


/**
 * 用户 业务层处理
 * 
 * @author ruoyi
 */
@Service
public class SysUserServiceImpl implements ISysUserService
{
    private static final Logger log = LoggerFactory.getLogger(SysUserServiceImpl.class);

    @Autowired
    private SysUserMapper userMapper;

    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysPostMapper postMapper;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    @Autowired
    private SysUserPostMapper userPostMapper;

    @Autowired
    private ISysConfigService configService;
    @Resource
    private ISysDeptService deptService;

    /**
     * 根据条件分页查询用户列表
     * 
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<SysUser> selectUserList(SysUser user)
    {
        return userMapper.selectUserList(user);
    }

    /**
     * 通过用户名查询用户
     * 
     * @param userName 用户名
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserByUserName(String userName)
    {
        return userMapper.selectUserByUserName(userName);
    }

    /**
     * 通过用户ID查询用户
     * 
     * @param id 用户ID
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserById(Long id)
    {
        return userMapper.selectUserById(id);
    }

    @Override
    public AjaxResult updateUnionId(SysUser user) {
        try {
            if(userMapper.updateUnionId(user) > 0) {
                return AjaxResult.successZero("");
            }
        }catch (Exception e) {
            return AjaxResult.error(-1, "保存异常");
        }

        return AjaxResult.error(-1, "保存失败");
    }

    /**
     * 查询用户所属角色组
     * 
     * @param userName 用户名
     * @return 结果
     */
    @Override
    public String selectUserRoleGroup(String userName)
    {
        List<SysRole> list = roleMapper.selectRolesByUserName(userName);
        StringBuffer idsStr = new StringBuffer();
        for (SysRole role : list)
        {
            idsStr.append(role.getRoleName()).append(",");
        }
        if (StringUtils.isNotEmpty(idsStr.toString()))
        {
            return idsStr.substring(0, idsStr.length() - 1);
        }
        return idsStr.toString();
    }

    /**
     * 查询用户所属岗位组
     * 
     * @param userName 用户名
     * @return 结果
     */
    @Override
    public String selectUserPostGroup(String userName)
    {
        List<SysPost> list = postMapper.selectPostsByUserName(userName);
        StringBuffer idsStr = new StringBuffer();
        for (SysPost post : list)
        {
            idsStr.append(post.getPostName()).append(",");
        }
        if (StringUtils.isNotEmpty(idsStr.toString()))
        {
            return idsStr.substring(0, idsStr.length() - 1);
        }
        return idsStr.toString();
    }

    /**
     * 校验用户名称是否唯一
     * 
     * @param userName 用户名称
     * @return 结果
     */
    @Override
    public String checkUserNameUnique(String userName)
    {
        int count = userMapper.checkUserNameUnique(userName);
        if (count > 0)
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验用户名称是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public String checkPhoneUnique(SysUser user)
    {
        Long id = StringUtils.isNull(user.getId()) ? -1L : user.getId();
        SysUser info = userMapper.checkPhoneUnique(user.getPhonenumber());
        if (StringUtils.isNotNull(info) && info.getId().longValue() != id.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验email是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public String checkEmailUnique(SysUser user)
    {
        Long id = StringUtils.isNull(user.getId()) ? -1L : user.getId();
        SysUser info = userMapper.checkEmailUnique(user.getEmail());
        if (StringUtils.isNotNull(info) && info.getId().longValue() != id.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验用户是否允许操作
     * 
     * @param user 用户信息
     */
    @Override
    public void checkUserAllowed(SysUser user)
    {
        if (StringUtils.isNotNull(user.getId()) && user.isAdmin())
        {
            throw new CustomException("不允许操作超级管理员用户");
        }
    }

    /**
     * 新增保存用户信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional
    public int insertUser(SysUser user)
    {
        // 新增用户信息
        int rows = userMapper.insertUser(user);
        // 新增用户岗位关联
        insertUserPost(user);
        // 新增用户与角色管理
        insertUserRole(user);
        return rows;
    }

    /**
     * 修改保存用户信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional
    public int updateUser(SysUser user)
    {
        Long id = user.getId();
        // 删除用户与角色关联
        userRoleMapper.deleteUserRoleByUserId(id);
        // 新增用户与角色管理
        insertUserRole(user);
        // 删除用户与岗位关联
        userPostMapper.deleteUserPostByUserId(id);
        // 新增用户与岗位管理
        insertUserPost(user);
        return userMapper.updateUser(user);
    }

    /**
     * 修改用户状态
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserStatus(SysUser user)
    {
        return userMapper.updateUser(user);
    }

    /**
     * 修改用户基本信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserProfile(SysUser user)
    {
        return userMapper.updateUser(user);
    }

    /**
     * 修改用户头像
     * 
     * @param userName 用户名
     * @param avatar 头像地址
     * @return 结果
     */
    @Override
    public boolean updateUserAvatar(String userName, String avatar)
    {
        return userMapper.updateUserAvatar(userName, avatar) > 0;
    }

    /**
     * 重置用户密码
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int resetPwd(SysUser user)
    {
        return userMapper.updateUser(user);
    }

    /**
     * 重置用户密码
     * 
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    @Override
    public int resetUserPwd(String userName, String password)
    {
        return userMapper.resetUserPwd(userName, password);
    }

    /**
     * 新增用户角色信息
     * 
     * @param user 用户对象
     */
    public void insertUserRole(SysUser user)
    {
        Long[] roles = user.getRoleIds();
        if (StringUtils.isNotNull(roles))
        {
            // 新增用户与角色管理
            List<SysUserRole> list = new ArrayList<SysUserRole>();
            for (Long roleId : roles)
            {
                SysUserRole ur = new SysUserRole();
                ur.setUserId(user.getId());
                ur.setRoleId(roleId);
                list.add(ur);
            }
            if (list.size() > 0)
            {
                userRoleMapper.batchUserRole(list);
            }
        }
    }

    /**
     * 新增用户岗位信息
     * 
     * @param user 用户对象
     */
    public void insertUserPost(SysUser user)
    {
        Long[] posts = user.getPostIds();
        if (StringUtils.isNotNull(posts))
        {
            // 新增用户与岗位管理
            List<SysUserPost> list = new ArrayList<SysUserPost>();
            for (Long postId : posts)
            {
                SysUserPost up = new SysUserPost();
                up.setUserId(user.getId());
                up.setPostId(postId);
                list.add(up);
            }
            if (list.size() > 0)
            {
                userPostMapper.batchUserPost(list);
            }
        }
    }

    /**
     * 通过用户ID删除用户
     * 
     * @param id 用户ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteUserById(Long id)
    {
        // 删除用户与角色关联
        userRoleMapper.deleteUserRoleByUserId(id);
        // 删除用户与岗位表
        userPostMapper.deleteUserPostByUserId(id);
        return userMapper.deleteUserById(id);
    }

    /**
     * 批量删除用户信息
     * 
     * @param ids 需要删除的用户ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteUserByIds(Long[] ids)
    {
        for (Long id : ids)
        {
            checkUserAllowed(new SysUser(id));
        }
        // 删除用户与角色关联
        userRoleMapper.deleteUserRole(ids);
        // 删除用户与岗位关联
        userPostMapper.deleteUserPost(ids);
        return userMapper.deleteUserByIds(ids);
    }

    /**
     * 导入用户数据
     * 
     * @param userList 用户数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName 操作用户
     * @return 结果
     */
    @Override
    public String importUser(List<SysUser> userList, Boolean isUpdateSupport, String operName)
    {
        if (StringUtils.isNull(userList) || userList.size() == 0)
        {
            throw new CustomException("导入用户数据不能为空！");
        }
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        String password = configService.selectConfigByKey("sys.user.initPassword");
        for (SysUser user : userList)
        {
            try
            {
                // 验证是否存在这个用户
                SysUser u = userMapper.selectUserByUserName(user.getUserName());
                if (StringUtils.isNull(u))
                {
                    user.setPassword(SecurityUtils.encryptPassword(password));
                    user.setCreateBy(operName);
                    this.insertUser(user);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、账号 " + user.getUserName() + " 导入成功");
                }
                else if (isUpdateSupport)
                {
                    user.setUpdateBy(operName);
                    this.updateUser(user);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、账号 " + user.getUserName() + " 更新成功");
                }
                else
                {
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、账号 " + user.getUserName() + " 已存在");
                }
            }
            catch (Exception e)
            {
                failureNum++;
                String msg = "<br/>" + failureNum + "、账号 " + user.getUserName() + " 导入失败：";
                failureMsg.append(msg + e.getMessage());
                log.error(msg, e);
            }
        }
        if (failureNum > 0)
        {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new CustomException(failureMsg.toString());
        }
        else
        {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
    }

    /**
     * 接收完整的用户对象提取用户的部门名称和角色名称
     * @param sysUserList
     * @return
     */
    @Override
    public AjaxResult getDeptNameAndRoleName(List<SysUser> sysUserList) {
        List<String> deptNameList = new ArrayList<>();
        List<String> nikeNameList = new ArrayList<>();
        if (sysUserList.size() > 0){
            for (SysUser user : sysUserList) {
                SysDept dept = user.getDept();
                if (ObjectUtils.isNotEmpty(dept)){
                    String deptName = dept.getDeptName();
                    deptNameList.add(deptName);
                }
                nikeNameList.add(user.getNickName());
            }
        }
        return AjaxResult.success().put("deptNameList",deptNameList).put("nikeNameList",nikeNameList);
    }

    /**
     * 部门下所有人
     */
    @Override
    public List<SysUser> listByDeptId(List<Long> deptIdList) {
        if (ObjectUtils.isEmpty(deptIdList)) {
            return Collections.emptyList();
        }

        return userMapper.listByDeptId(deptIdList);
    }


    /**
     * 部门下所有人
     */
    @Override
    public List<SysUser> listByDeptIdList(List<Long> deptIdList) {
        if (ObjectUtils.isEmpty(deptIdList)) {
            return Collections.emptyList();
        }

        return userMapper.listByDeptIdList(deptIdList);
    }

    /**
     * 为用户档案定制新增接口
     * @param result
     * @return
     */
    @Override
    public void addUser(List<JSONObject> result) {
        JSONObject jsonObject = result.get(0);
        String nameUser = String.valueOf(jsonObject.get("name_user"));
        String genderUser = String.valueOf(jsonObject.get("gender_user"));
        String callNumber = String.valueOf(jsonObject.get("call_number"));
        SysUser user = new SysUser();
        user.setPhonenumber(callNumber);
        user.setSex("男".equals(genderUser)?"0":"1");
        user.setUserName(callNumber);
        user.setNickName(nameUser);
        user.setStatus("1");
        user.setCreateBy("admin");
        user.setCreateTime(DateUtils.getNowDate());
        user.setPassword(SecurityUtils.encryptPassword(callNumber));
        if (!UserConstants.NOT_UNIQUE.equals(checkPhoneUnique(user))){
            userMapper.insertUser(user);
        }
    }

    /**
     * 功能描述: 树形结构，部门下的人员
     *
     * @param dept dept
     * @return java.util.List<com.boman.domain.SysDept>
     */
    @Override
    public List<SysDept> listUserTree(SysDept dept) {
        List<SysDept> sysDepts = deptService.selectDeptList(dept);
        List<Long> deptIdList = ObjectUtils.map(sysDepts, SysDept::getId);
        List<SysUser> sysUsers = listByDeptIdList(deptIdList);

        for (SysDept sysDept : sysDepts) {
            List<SysUser> users = new ArrayList<>(16);
            for (SysUser sysUser : sysUsers) {
                if (sysDept.getId().equals(sysUser.getDeptId())) {
                    // 为了保持树形结构的完整性，需要外层和内层属性名称一致，这里都为 deptName （王梦韦）
                    sysUser.setDeptName(sysUser.getUserName());
                    users.add(sysUser);
                }
            }
            sysDept.setUsers(users);
        }

        return sysDepts;
    }

}
